# This is spectacularly generated code by spectacular v0.0.0 based on
# Qlik Cloud Services 0.348.2

from __future__ import annotations

from dataclasses import asdict, dataclass
from typing import Union

from ..auth import Auth, Config
from ..listable import ListableResource


@dataclass
class User:
    """

    Attributes
    ----------
    id: str
      The unique user identifier.
    name: str
      The name of the user.
    subject: str
      The unique user identitier from an identity provider.
    tenantId: str
      The tenant that the user belongs too.
    assignedGroups: list[object]
    assignedRoles: list[object]
    created: str
      Deprecated. Use `createdAt` instead.
    createdAt: str
      The timestamp for when the user record was created.
    email: str
      The email address for the user.
    inviteExpiry: float
      The number of seconds until the user invitation will expire.
    lastUpdated: str
      Deprecated. Use `lastUpdatedAt` instead.
    lastUpdatedAt: str
      The timestamp for when the user record was last updated.
    links: object
    locale: str
      Represents the end-user's language tag.
    picture: str
      A static url linking to the avatar of the user.
    preferredLocale: str
      Represents the end-user's preferred language tag.
    preferredZoneinfo: str
      Represents the end-user's preferred time zone.
    roles: list[str]
      List of system roles to which the user has been assigned. Only returned when permitted by access control. Deprecated. Use `assignedRoles` instead.
    status: str
      The status of the user within the tenant.
    zoneinfo: str
      Represents the end-user's time zone.
    """

    id: str = None
    name: str = None
    subject: str = None
    tenantId: str = None
    assignedGroups: list[object] = None
    assignedRoles: list[object] = None
    created: str = None
    createdAt: str = None
    email: str = None
    inviteExpiry: float = None
    lastUpdated: str = None
    lastUpdatedAt: str = None
    links: object = None
    locale: str = None
    picture: str = None
    preferredLocale: str = None
    preferredZoneinfo: str = None
    roles: list[str] = None
    status: str = None
    zoneinfo: str = None

    def __init__(self_, **kvargs):
        if "id" in kvargs:
            if type(kvargs["id"]).__name__ is self_.__annotations__["id"]:
                self_.id = kvargs["id"]
            else:
                self_.id = kvargs["id"]
        if "name" in kvargs:
            if type(kvargs["name"]).__name__ is self_.__annotations__["name"]:
                self_.name = kvargs["name"]
            else:
                self_.name = kvargs["name"]
        if "subject" in kvargs:
            if type(kvargs["subject"]).__name__ is self_.__annotations__["subject"]:
                self_.subject = kvargs["subject"]
            else:
                self_.subject = kvargs["subject"]
        if "tenantId" in kvargs:
            if type(kvargs["tenantId"]).__name__ is self_.__annotations__["tenantId"]:
                self_.tenantId = kvargs["tenantId"]
            else:
                self_.tenantId = kvargs["tenantId"]
        if "assignedGroups" in kvargs:
            if (
                type(kvargs["assignedGroups"]).__name__
                is self_.__annotations__["assignedGroups"]
            ):
                self_.assignedGroups = kvargs["assignedGroups"]
            else:
                self_.assignedGroups = kvargs["assignedGroups"]
        if "assignedRoles" in kvargs:
            if (
                type(kvargs["assignedRoles"]).__name__
                is self_.__annotations__["assignedRoles"]
            ):
                self_.assignedRoles = kvargs["assignedRoles"]
            else:
                self_.assignedRoles = kvargs["assignedRoles"]
        if "created" in kvargs:
            if type(kvargs["created"]).__name__ is self_.__annotations__["created"]:
                self_.created = kvargs["created"]
            else:
                self_.created = kvargs["created"]
        if "createdAt" in kvargs:
            if type(kvargs["createdAt"]).__name__ is self_.__annotations__["createdAt"]:
                self_.createdAt = kvargs["createdAt"]
            else:
                self_.createdAt = kvargs["createdAt"]
        if "email" in kvargs:
            if type(kvargs["email"]).__name__ is self_.__annotations__["email"]:
                self_.email = kvargs["email"]
            else:
                self_.email = kvargs["email"]
        if "inviteExpiry" in kvargs:
            if (
                type(kvargs["inviteExpiry"]).__name__
                is self_.__annotations__["inviteExpiry"]
            ):
                self_.inviteExpiry = kvargs["inviteExpiry"]
            else:
                self_.inviteExpiry = kvargs["inviteExpiry"]
        if "lastUpdated" in kvargs:
            if (
                type(kvargs["lastUpdated"]).__name__
                is self_.__annotations__["lastUpdated"]
            ):
                self_.lastUpdated = kvargs["lastUpdated"]
            else:
                self_.lastUpdated = kvargs["lastUpdated"]
        if "lastUpdatedAt" in kvargs:
            if (
                type(kvargs["lastUpdatedAt"]).__name__
                is self_.__annotations__["lastUpdatedAt"]
            ):
                self_.lastUpdatedAt = kvargs["lastUpdatedAt"]
            else:
                self_.lastUpdatedAt = kvargs["lastUpdatedAt"]
        if "links" in kvargs:
            if type(kvargs["links"]).__name__ is self_.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = kvargs["links"]
        if "locale" in kvargs:
            if type(kvargs["locale"]).__name__ is self_.__annotations__["locale"]:
                self_.locale = kvargs["locale"]
            else:
                self_.locale = kvargs["locale"]
        if "picture" in kvargs:
            if type(kvargs["picture"]).__name__ is self_.__annotations__["picture"]:
                self_.picture = kvargs["picture"]
            else:
                self_.picture = kvargs["picture"]
        if "preferredLocale" in kvargs:
            if (
                type(kvargs["preferredLocale"]).__name__
                is self_.__annotations__["preferredLocale"]
            ):
                self_.preferredLocale = kvargs["preferredLocale"]
            else:
                self_.preferredLocale = kvargs["preferredLocale"]
        if "preferredZoneinfo" in kvargs:
            if (
                type(kvargs["preferredZoneinfo"]).__name__
                is self_.__annotations__["preferredZoneinfo"]
            ):
                self_.preferredZoneinfo = kvargs["preferredZoneinfo"]
            else:
                self_.preferredZoneinfo = kvargs["preferredZoneinfo"]
        if "roles" in kvargs:
            if type(kvargs["roles"]).__name__ is self_.__annotations__["roles"]:
                self_.roles = kvargs["roles"]
            else:
                self_.roles = kvargs["roles"]
        if "status" in kvargs:
            if type(kvargs["status"]).__name__ is self_.__annotations__["status"]:
                self_.status = kvargs["status"]
            else:
                self_.status = kvargs["status"]
        if "zoneinfo" in kvargs:
            if type(kvargs["zoneinfo"]).__name__ is self_.__annotations__["zoneinfo"]:
                self_.zoneinfo = kvargs["zoneinfo"]
            else:
                self_.zoneinfo = kvargs["zoneinfo"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)

    def delete(self) -> None:
        """
        Deletes a user resource.

        Parameters
        ----------
        """

        self.auth.rest(
            path="/users/{userId}".replace("{userId}", self.id),
            method="DELETE",
            params={},
            data=None,
        )

    def patch(self, data: UserJSONPatchArray) -> None:
        """
        Updates a user resource.

        Parameters
        ----------
        data: UserJSONPatchArray
        """

        try:
            data = asdict(data)
        except:
            data = data

        self.auth.rest(
            path="/users/{userId}".replace("{userId}", self.id),
            method="PATCH",
            params={},
            data=data,
        )


@dataclass
class Filter:
    """

    Attributes
    ----------
    filter: str
      The advanced filtering to be applied the query. All conditional statements within this query parameter are case insensitive.
    """

    filter: str = None

    def __init__(self_, **kvargs):
        if "filter" in kvargs:
            if type(kvargs["filter"]).__name__ is self_.__annotations__["filter"]:
                self_.filter = kvargs["filter"]
            else:
                self_.filter = kvargs["filter"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class Metadata:
    """

    Attributes
    ----------
    valid_roles: list[str]
      List of system roles to which the user can be assigned.
    """

    valid_roles: list[str] = None

    def __init__(self_, **kvargs):
        if "valid_roles" in kvargs:
            if (
                type(kvargs["valid_roles"]).__name__
                is self_.__annotations__["valid_roles"]
            ):
                self_.valid_roles = kvargs["valid_roles"]
            else:
                self_.valid_roles = kvargs["valid_roles"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UserCount:
    """

    Attributes
    ----------
    total: float
    """

    total: float = None

    def __init__(self_, **kvargs):
        if "total" in kvargs:
            if type(kvargs["total"]).__name__ is self_.__annotations__["total"]:
                self_.total = kvargs["total"]
            else:
                self_.total = kvargs["total"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UserJSONPatch:
    """
    A JSON Patch document as defined in http://tools.ietf.org/html/rfc6902.

    Attributes
    ----------
    op: str
      The operation to be performed.
    path: str
      A JSON Pointer.
    value: Union[str,bool,list]
      The value to be used for this operation.
    """

    op: str = None
    path: str = None
    value: Union[str, bool, list] = None

    def __init__(self_, **kvargs):
        if "op" in kvargs:
            if type(kvargs["op"]).__name__ is self_.__annotations__["op"]:
                self_.op = kvargs["op"]
            else:
                self_.op = kvargs["op"]
        if "path" in kvargs:
            if type(kvargs["path"]).__name__ is self_.__annotations__["path"]:
                self_.path = kvargs["path"]
            else:
                self_.path = kvargs["path"]
        if "value" in kvargs:
            if type(kvargs["value"]).__name__ is self_.__annotations__["value"]:
                self_.value = kvargs["value"]
            else:
                self_.value = kvargs["value"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UserJSONPatchArray:
    """
    An array of JSON Patch documents

    Attributes
    ----------
    """

    def __init__(self_, **kvargs):
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UserPostSchema:
    """

    Attributes
    ----------
    subject: str
      The unique user identitier from an identity provider.
    assignedGroups: list[object]
    assignedRoles: list[object]
    email: str
      The email address for the user. This is a required field when inviting a user.
    name: str
      The name of the user.
    picture: str
      A static url linking to the avatar of the user.
    roles: list[str]
      List of system roles to which the user has been assigned. Only returned when permitted by access control.
    status: str
      The status of the created user within the tenant.
    tenantId: str
      The tenant that the user will belong too.
    """

    subject: str = None
    assignedGroups: list[object] = None
    assignedRoles: list[object] = None
    email: str = None
    name: str = None
    picture: str = None
    roles: list[str] = None
    status: str = None
    tenantId: str = None

    def __init__(self_, **kvargs):
        if "subject" in kvargs:
            if type(kvargs["subject"]).__name__ is self_.__annotations__["subject"]:
                self_.subject = kvargs["subject"]
            else:
                self_.subject = kvargs["subject"]
        if "assignedGroups" in kvargs:
            if (
                type(kvargs["assignedGroups"]).__name__
                is self_.__annotations__["assignedGroups"]
            ):
                self_.assignedGroups = kvargs["assignedGroups"]
            else:
                self_.assignedGroups = kvargs["assignedGroups"]
        if "assignedRoles" in kvargs:
            if (
                type(kvargs["assignedRoles"]).__name__
                is self_.__annotations__["assignedRoles"]
            ):
                self_.assignedRoles = kvargs["assignedRoles"]
            else:
                self_.assignedRoles = kvargs["assignedRoles"]
        if "email" in kvargs:
            if type(kvargs["email"]).__name__ is self_.__annotations__["email"]:
                self_.email = kvargs["email"]
            else:
                self_.email = kvargs["email"]
        if "name" in kvargs:
            if type(kvargs["name"]).__name__ is self_.__annotations__["name"]:
                self_.name = kvargs["name"]
            else:
                self_.name = kvargs["name"]
        if "picture" in kvargs:
            if type(kvargs["picture"]).__name__ is self_.__annotations__["picture"]:
                self_.picture = kvargs["picture"]
            else:
                self_.picture = kvargs["picture"]
        if "roles" in kvargs:
            if type(kvargs["roles"]).__name__ is self_.__annotations__["roles"]:
                self_.roles = kvargs["roles"]
            else:
                self_.roles = kvargs["roles"]
        if "status" in kvargs:
            if type(kvargs["status"]).__name__ is self_.__annotations__["status"]:
                self_.status = kvargs["status"]
            else:
                self_.status = kvargs["status"]
        if "tenantId" in kvargs:
            if type(kvargs["tenantId"]).__name__ is self_.__annotations__["tenantId"]:
                self_.tenantId = kvargs["tenantId"]
            else:
                self_.tenantId = kvargs["tenantId"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UsersClass:
    """

    Attributes
    ----------
    data: list[User]
    links: object
    """

    data: list[User] = None
    links: object = None

    def __init__(self_, **kvargs):
        if "data" in kvargs:
            if type(kvargs["data"]).__name__ is self_.__annotations__["data"]:
                self_.data = kvargs["data"]
            else:
                self_.data = [User(**e) for e in kvargs["data"]]
        if "links" in kvargs:
            if type(kvargs["links"]).__name__ is self_.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = kvargs["links"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


class Users:
    def __init__(self, config: Config) -> None:
        self.config = config
        self.auth = Auth(config)

    def get_users(
        self,
        tenantId: str = None,
        subject: str = None,
        email: str = None,
        status: str = "active",
        role: str = None,
        fields: str = None,
        limit: float = 20,
        sort: str = "+name",
        sortBy: str = "name",
        sortOrder: str = "asc",
        startingAfter: str = None,
        endingBefore: str = None,
        filter: str = None,
        next: str = None,
        prev: str = None,
        max_items: int = 20,
    ) -> ListableResource[User]:
        """
        Retrieves a list of users matching the query.


        tenantId: str
          The tenant ID to filter by. Deprecated. Use the new `filter` parameter to provide an advanced query filter.

        subject: str
          The subject to filter by. Deprecated. Use the new `filter` parameter to provide an advanced query filter.

        email: str
          The email to filter by. Deprecated. Use the new `filter` parameter to provide an advanced query filter.

        status: str
          The status to filter by. Supports multiple values delimited by commas. Deprecated. Use the new `filter` parameter to provide an advanced query filter.

        role: str
          The role to filter by. Deprecated. Use the new `filter` parameter to provide an advanced query filter.

        fields: str
          A comma-delimited string of the requested fields per entity. If the 'links' value is omitted, then the entity HATEOAS link will also be omitted.

        limit: float
          The number of user entries to retrieve.

        sort: str
          The field to sort by, with +/- prefix indicating sort order

        sortBy: str
          The user parameter to sort by. Deprecated. Use `sort` instead.

        sortOrder: str
          The sort order, either ascending or descending. Deprecated. Use `sort` instead.

        startingAfter: str
          Get users with IDs that are higher than the target user ID. Cannot be used in conjunction with endingBefore. Deprecated. Use `next` instead.

        endingBefore: str
          Get users with IDs that are lower than the target user ID. Cannot be used in conjunction with startingAfter. Deprecated. Use `prev` instead.

        filter: str
          The advanced filtering to use for the query. Refer to [RFC 7644](https://datatracker.ietf.org/doc/rfc7644/) for the syntax. Cannot be combined with any of the fields marked as deprecated. All conditional statements within this query parameter are case insensitive.

          The following fields support the `eq` operator: `id`, `subject`, `name`, `email`, `status`, `clientId`, `assignedRoles.id` `assignedRoles.name`, `assignedGroups.id`, `assignedGroupsAssignedRoles.name`

          Additionally, the following fields support the `co` operator: `name`, `email`, `subject`

          Queries may be rate limited if they differ greatly from these examples:

          ```
          (id eq "62716ab404a7bd8626af9bd6" or id eq "62716ac4c7e500e13ff5fa22") and (status eq "active" or status eq "disabled")
          ```

          ```
          name co "query" or email co "query" or subject co "query" or id eq "query" or assignedRoles.name eq "query"
          ```

        next: str
          Get users that come after this cursor value when sorted. Cannot be used in conjunction with `prev`.

        prev: str
          Get users that come before this cursor value when sorted. Cannot be used in conjunction with `next`.

        Parameters
        ----------
        tenantId: str = None
        subject: str = None
        email: str = None
        status: str = "active"
        role: str = None
        fields: str = None
        limit: float = 20
        sort: str = "+name"
        sortBy: str = "name"
        sortOrder: str = "asc"
        startingAfter: str = None
        endingBefore: str = None
        filter: str = None
        next: str = None
        prev: str = None
        """
        query_params = {}
        if tenantId is not None:
            query_params["tenantId"] = tenantId
        if subject is not None:
            query_params["subject"] = subject
        if email is not None:
            query_params["email"] = email
        if status is not None:
            query_params["status"] = status
        if role is not None:
            query_params["role"] = role
        if fields is not None:
            query_params["fields"] = fields
        if limit is not None:
            query_params["limit"] = limit
        if sort is not None:
            query_params["sort"] = sort
        if sortBy is not None:
            query_params["sortBy"] = sortBy
        if sortOrder is not None:
            query_params["sortOrder"] = sortOrder
        if startingAfter is not None:
            query_params["startingAfter"] = startingAfter
        if endingBefore is not None:
            query_params["endingBefore"] = endingBefore
        if filter is not None:
            query_params["filter"] = filter
        if next is not None:
            query_params["next"] = next
        if prev is not None:
            query_params["prev"] = prev

        response = self.auth.rest(
            path="/users",
            method="GET",
            params=query_params,
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=User,
            auth=self.auth,
            path="/users",
            max_items=max_items,
            query_params=query_params,
        )

    def create(self, data: UserPostSchema) -> User:
        """
        Creates a user in a given tenant.
        Creates and returns a user in a given tenant. Note that only creating invited status users is supported for users of the given tenant.


        Authorization: str
          The JWT used for authentication. Send the JWT in the Authorization header using the Bearer schema.

        Parameters
        ----------
        data: UserPostSchema
        """

        try:
            data = asdict(data)
        except:
            data = data

        response = self.auth.rest(
            path="/users",
            method="POST",
            params={},
            data=data,
        )
        obj = User(**response.json())
        obj.auth = self.auth
        return obj

    def count(self, tenantId: str = None) -> UserCount:
        """
        Returns the number of users in a given tenant


        tenantId: str
          The tenant ID to filter by.

        Parameters
        ----------
        tenantId: str = None
        """
        query_params = {}
        if tenantId is not None:
            query_params["tenantId"] = tenantId

        response = self.auth.rest(
            path="/users/actions/count",
            method="GET",
            params=query_params,
            data=None,
        )
        obj = UserCount(**response.json())
        obj.auth = self.auth
        return obj

    def filter(
        self,
        data: Filter,
        fields: str = None,
        sort: str = "+name",
        next: str = None,
        prev: str = None,
        limit: float = 20,
        max_items: int = 20,
    ) -> ListableResource[User]:
        """
        Retrieves a list of users matching the filter using advanced query string.


        fields: str
          A comma-delimited string of the requested fields per entity. If the 'links' value is omitted, then the entity HATEOAS link will also be omitted.

        sort: str
          The field to sort by, with +/- prefix indicating sort order

        next: str
          Get users with IDs that are higher than the target user ID. Cannot be used in conjunction with prev.

        prev: str
          Get users with IDs that are lower than the target user ID. Cannot be used in conjunction with next.

        limit: float
          The number of user entries to retrieve.

        Parameters
        ----------
        fields: str = None
        sort: str = "+name"
        next: str = None
        prev: str = None
        limit: float = 20
        data: Filter
        """
        query_params = {}
        if fields is not None:
            query_params["fields"] = fields
        if sort is not None:
            query_params["sort"] = sort
        if next is not None:
            query_params["next"] = next
        if prev is not None:
            query_params["prev"] = prev
        if limit is not None:
            query_params["limit"] = limit

        try:
            data = asdict(data)
        except:
            data = data

        response = self.auth.rest(
            path="/users/actions/filter",
            method="POST",
            params=query_params,
            data=data,
        )
        return ListableResource(
            response=response.json(),
            cls=User,
            auth=self.auth,
            path="/users/actions/filter",
            max_items=max_items,
            query_params=query_params,
        )

    def get_me(self) -> any:
        """
        Redirects to retrieve the user resource associated with the JWT claims.


        Parameters
        ----------
        """

        response = self.auth.rest(
            path="/users/me",
            method="GET",
            params={},
            data=None,
        )
        return response.json()

    def get_metadata(self) -> Metadata:
        """
        Returns the metadata with regard to the user configuration.
        Deprecated, use GET /v1/roles instead.


        Parameters
        ----------
        """

        response = self.auth.rest(
            path="/users/metadata",
            method="GET",
            params={},
            data=None,
        )
        obj = Metadata(**response.json())
        obj.auth = self.auth
        return obj

    def get(self, userId: str) -> User:
        """
        Returns the requested user.
        Returns the requested user.


        userId: str
          The user's unique identifier

        Parameters
        ----------
        userId: str
        """

        response = self.auth.rest(
            path="/users/{userId}".replace("{userId}", userId),
            method="GET",
            params={},
            data=None,
        )
        obj = User(**response.json())
        obj.auth = self.auth
        return obj



from ._version import __version__  # noqa
from .auth import Auth  # noqa
from .auth_type import AuthType  # noqa
from .config import Config  # noqa
from .generate_signed_token import generate_signed_token  # noqa
from .generated.Apps import *  # noqa
from .generated.Extensions import *  # noqa

# expose from generated
from .generated.Items import *  # noqa
from .generated.Qix import *  # noqa
from .generated.Qix_Datafiles import *  # noqa
from .generated.Reloads import *  # noqa
from .generated.Spaces import *  # noqa
from .generated.Themes import *  # noqa
from .generated.Users import *  # noqa
from .qlik import Qlik  # noqa
from .rpc import RequestInterceptor, RequestObject, ResponseInterceptor  # noqa



# This is spectacularly generated code by spectacular v0.0.0 based on
# Qlik Cloud Services 0.348.2

from __future__ import annotations

import warnings
from dataclasses import asdict, dataclass

from ..auth import Auth, Config
from ..listable import ListableResource


@dataclass
class Space:
    """
    A space is a security context simplifying the management of access control by allowing users to control it on the containers instead of on the resources themselves.

    Attributes
    ----------
    id: str
      A unique identifier for the space, for example, 62716f4b39b865ece543cd45.
    links: object
    name: str
      The name of the space. Personal spaces do not have a name.
    tenantId: str
      The ID for the tenant, for example, xqGQ0k66vSR8f9G7J-vYtHZQkiYrCpct.
    createdAt: str
      The date and time when the space was created.
    createdBy: str
      The ID of the user who created the space.
    description: str
      The description of the space. Personal spaces do not have a description.
    meta: object
      Information about the space settings.
    ownerId: str
      The ID for the space owner.
    type: str
      The type of space such as shared, managed, and so on.
    updatedAt: str
      The date and time when the space was updated.
    """

    id: str = None
    links: object = None
    name: str = None
    tenantId: str = None
    createdAt: str = None
    createdBy: str = None
    description: str = None
    meta: object = None
    ownerId: str = None
    type: str = None
    updatedAt: str = None

    def __init__(self_, **kvargs):
        if "id" in kvargs:
            if type(kvargs["id"]).__name__ is self_.__annotations__["id"]:
                self_.id = kvargs["id"]
            else:
                self_.id = kvargs["id"]
        if "links" in kvargs:
            if type(kvargs["links"]).__name__ is self_.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = kvargs["links"]
        if "name" in kvargs:
            if type(kvargs["name"]).__name__ is self_.__annotations__["name"]:
                self_.name = kvargs["name"]
            else:
                self_.name = kvargs["name"]
        if "tenantId" in kvargs:
            if type(kvargs["tenantId"]).__name__ is self_.__annotations__["tenantId"]:
                self_.tenantId = kvargs["tenantId"]
            else:
                self_.tenantId = kvargs["tenantId"]
        if "createdAt" in kvargs:
            if type(kvargs["createdAt"]).__name__ is self_.__annotations__["createdAt"]:
                self_.createdAt = kvargs["createdAt"]
            else:
                self_.createdAt = kvargs["createdAt"]
        if "createdBy" in kvargs:
            if type(kvargs["createdBy"]).__name__ is self_.__annotations__["createdBy"]:
                self_.createdBy = kvargs["createdBy"]
            else:
                self_.createdBy = kvargs["createdBy"]
        if "description" in kvargs:
            if (
                type(kvargs["description"]).__name__
                is self_.__annotations__["description"]
            ):
                self_.description = kvargs["description"]
            else:
                self_.description = kvargs["description"]
        if "meta" in kvargs:
            if type(kvargs["meta"]).__name__ is self_.__annotations__["meta"]:
                self_.meta = kvargs["meta"]
            else:
                self_.meta = kvargs["meta"]
        if "ownerId" in kvargs:
            if type(kvargs["ownerId"]).__name__ is self_.__annotations__["ownerId"]:
                self_.ownerId = kvargs["ownerId"]
            else:
                self_.ownerId = kvargs["ownerId"]
        if "type" in kvargs:
            if type(kvargs["type"]).__name__ is self_.__annotations__["type"]:
                self_.type = kvargs["type"]
            else:
                self_.type = kvargs["type"]
        if "updatedAt" in kvargs:
            if type(kvargs["updatedAt"]).__name__ is self_.__annotations__["updatedAt"]:
                self_.updatedAt = kvargs["updatedAt"]
            else:
                self_.updatedAt = kvargs["updatedAt"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)

    def delete(self) -> None:
        """
        Deletes a space.

        Parameters
        ----------
        """

        self.auth.rest(
            path="/spaces/{spaceId}".replace("{spaceId}", self.id),
            method="DELETE",
            params={},
            data=None,
        )

    def patch(self, data: SpacePatch) -> Space:
        """
        Experimental
        Patches (updates) a space (partially).

        Parameters
        ----------
        data: SpacePatch
        """
        warnings.warn("patch is experimental", UserWarning, stacklevel=2)

        try:
            data = asdict(data)
        except:
            data = data

        response = self.auth.rest(
            path="/spaces/{spaceId}".replace("{spaceId}", self.id),
            method="PATCH",
            params={},
            data=data,
        )
        self.__init__(**response.json())
        return self

    def set(self, data: SpaceUpdate) -> Space:
        """
        Experimental
        Updates a space.

        Parameters
        ----------
        data: SpaceUpdate
        """
        warnings.warn("set is experimental", UserWarning, stacklevel=2)

        try:
            data = asdict(data)
        except:
            data = data

        response = self.auth.rest(
            path="/spaces/{spaceId}".replace("{spaceId}", self.id),
            method="PUT",
            params={},
            data=data,
        )
        self.__init__(**response.json())
        return self

    def get_assignments(
        self, limit: int = 10, next: str = None, prev: str = None, max_items: int = 10
    ) -> ListableResource[Assignment]:
        """
        Retrieves the assignments of the space matching the query.

        Parameters
        ----------
        limit: int = 10
        next: str = None
        prev: str = None
        """
        query_params = {}
        if limit is not None:
            query_params["limit"] = limit
        if next is not None:
            query_params["next"] = next
        if prev is not None:
            query_params["prev"] = prev

        response = self.auth.rest(
            path="/spaces/{spaceId}/assignments".replace("{spaceId}", self.id),
            method="GET",
            params=query_params,
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=Assignment,
            auth=self.auth,
            path="/spaces/{spaceId}/assignments".replace("{spaceId}", self.id),
            max_items=max_items,
            query_params=query_params,
        )

    def create_assignment(self, data: AssignmentCreate) -> Assignment:
        """
        Creates an assignment.

        Parameters
        ----------
        data: AssignmentCreate
        """

        try:
            data = asdict(data)
        except:
            data = data

        response = self.auth.rest(
            path="/spaces/{spaceId}/assignments".replace("{spaceId}", self.id),
            method="POST",
            params={},
            data=data,
        )
        obj = Assignment(**response.json())
        obj.auth = self.auth
        return obj

    def get_shares(
        self,
        name: str = None,
        resourceId: str = None,
        resourceType: str = None,
        groupId: str = None,
        userId: str = None,
        limit: int = 10,
        next: str = None,
        prev: str = None,
        max_items: int = 10,
    ) -> ListableResource[Share]:
        """
        Experimental
        Retrieves the shares of the space matching the query.

        Parameters
        ----------
        name: str = None
        resourceId: str = None
        resourceType: str = None
        groupId: str = None
        userId: str = None
        limit: int = 10
        next: str = None
        prev: str = None
        """
        warnings.warn("get_shares is experimental", UserWarning, stacklevel=2)
        query_params = {}
        if name is not None:
            query_params["name"] = name
        if resourceId is not None:
            query_params["resourceId"] = resourceId
        if resourceType is not None:
            query_params["resourceType"] = resourceType
        if groupId is not None:
            query_params["groupId"] = groupId
        if userId is not None:
            query_params["userId"] = userId
        if limit is not None:
            query_params["limit"] = limit
        if next is not None:
            query_params["next"] = next
        if prev is not None:
            query_params["prev"] = prev

        response = self.auth.rest(
            path="/spaces/{spaceId}/shares".replace("{spaceId}", self.id),
            method="GET",
            params=query_params,
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=Share,
            auth=self.auth,
            path="/spaces/{spaceId}/shares".replace("{spaceId}", self.id),
            max_items=max_items,
            query_params=query_params,
        )

    def create_share(self, data: ShareCreate) -> Share:
        """
        Experimental
        Creates a share.

        Parameters
        ----------
        data: ShareCreate
        """
        warnings.warn("create_share is experimental", UserWarning, stacklevel=2)

        try:
            data = asdict(data)
        except:
            data = data

        response = self.auth.rest(
            path="/spaces/{spaceId}/shares".replace("{spaceId}", self.id),
            method="POST",
            params={},
            data=data,
        )
        obj = Share(**response.json())
        obj.auth = self.auth
        return obj


@dataclass
class Assignment:
    """

    Attributes
    ----------
    assigneeId: str
      The userId or groupId based on the type.
    id: str
    links: object
    roles: list[str]
      The roles assigned to a user or group. Must not be empty.
    spaceId: str
      The unique identifier for the space.
    tenantId: str
      The unique identifier for the tenant.
    type: str
    createdAt: str
      The date and time when the space was created.
    createdBy: str
      The ID of the user who created the assignment.
    updatedAt: str
      The date and time when the space was updated.
    updatedBy: str
      The ID of the user who updated the assignment.
    """

    assigneeId: str = None
    id: str = None
    links: object = None
    roles: list[str] = None
    spaceId: str = None
    tenantId: str = None
    type: str = None
    createdAt: str = None
    createdBy: str = None
    updatedAt: str = None
    updatedBy: str = None

    def __init__(self_, **kvargs):
        if "assigneeId" in kvargs:
            if (
                type(kvargs["assigneeId"]).__name__
                is self_.__annotations__["assigneeId"]
            ):
                self_.assigneeId = kvargs["assigneeId"]
            else:
                self_.assigneeId = kvargs["assigneeId"]
        if "id" in kvargs:
            if type(kvargs["id"]).__name__ is self_.__annotations__["id"]:
                self_.id = kvargs["id"]
            else:
                self_.id = kvargs["id"]
        if "links" in kvargs:
            if type(kvargs["links"]).__name__ is self_.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = kvargs["links"]
        if "roles" in kvargs:
            if type(kvargs["roles"]).__name__ is self_.__annotations__["roles"]:
                self_.roles = kvargs["roles"]
            else:
                self_.roles = kvargs["roles"]
        if "spaceId" in kvargs:
            if type(kvargs["spaceId"]).__name__ is self_.__annotations__["spaceId"]:
                self_.spaceId = kvargs["spaceId"]
            else:
                self_.spaceId = kvargs["spaceId"]
        if "tenantId" in kvargs:
            if type(kvargs["tenantId"]).__name__ is self_.__annotations__["tenantId"]:
                self_.tenantId = kvargs["tenantId"]
            else:
                self_.tenantId = kvargs["tenantId"]
        if "type" in kvargs:
            if type(kvargs["type"]).__name__ is self_.__annotations__["type"]:
                self_.type = kvargs["type"]
            else:
                self_.type = kvargs["type"]
        if "createdAt" in kvargs:
            if type(kvargs["createdAt"]).__name__ is self_.__annotations__["createdAt"]:
                self_.createdAt = kvargs["createdAt"]
            else:
                self_.createdAt = kvargs["createdAt"]
        if "createdBy" in kvargs:
            if type(kvargs["createdBy"]).__name__ is self_.__annotations__["createdBy"]:
                self_.createdBy = kvargs["createdBy"]
            else:
                self_.createdBy = kvargs["createdBy"]
        if "updatedAt" in kvargs:
            if type(kvargs["updatedAt"]).__name__ is self_.__annotations__["updatedAt"]:
                self_.updatedAt = kvargs["updatedAt"]
            else:
                self_.updatedAt = kvargs["updatedAt"]
        if "updatedBy" in kvargs:
            if type(kvargs["updatedBy"]).__name__ is self_.__annotations__["updatedBy"]:
                self_.updatedBy = kvargs["updatedBy"]
            else:
                self_.updatedBy = kvargs["updatedBy"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class AssignmentCreate:
    """

    Attributes
    ----------
    assigneeId: str
      The userId or groupId based on the type.
    roles: list[str]
      The roles assigned to the assigneeId.
    type: str
      The type of space such as shared, managed, and so on.
    """

    assigneeId: str = None
    roles: list[str] = None
    type: str = None

    def __init__(self_, **kvargs):
        if "assigneeId" in kvargs:
            if (
                type(kvargs["assigneeId"]).__name__
                is self_.__annotations__["assigneeId"]
            ):
                self_.assigneeId = kvargs["assigneeId"]
            else:
                self_.assigneeId = kvargs["assigneeId"]
        if "roles" in kvargs:
            if type(kvargs["roles"]).__name__ is self_.__annotations__["roles"]:
                self_.roles = kvargs["roles"]
            else:
                self_.roles = kvargs["roles"]
        if "type" in kvargs:
            if type(kvargs["type"]).__name__ is self_.__annotations__["type"]:
                self_.type = kvargs["type"]
            else:
                self_.type = kvargs["type"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class AssignmentUpdate:
    """

    Attributes
    ----------
    roles: list[str]
      The roles assigned to the assigneeId.
    """

    roles: list[str] = None

    def __init__(self_, **kvargs):
        if "roles" in kvargs:
            if type(kvargs["roles"]).__name__ is self_.__annotations__["roles"]:
                self_.roles = kvargs["roles"]
            else:
                self_.roles = kvargs["roles"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class Assignments:
    """

    Attributes
    ----------
    data: list[Assignment]
    links: object
    meta: object
    """

    data: list[Assignment] = None
    links: object = None
    meta: object = None

    def __init__(self_, **kvargs):
        if "data" in kvargs:
            if type(kvargs["data"]).__name__ is self_.__annotations__["data"]:
                self_.data = kvargs["data"]
            else:
                self_.data = [Assignment(**e) for e in kvargs["data"]]
        if "links" in kvargs:
            if type(kvargs["links"]).__name__ is self_.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = kvargs["links"]
        if "meta" in kvargs:
            if type(kvargs["meta"]).__name__ is self_.__annotations__["meta"]:
                self_.meta = kvargs["meta"]
            else:
                self_.meta = kvargs["meta"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class FilterSpaces:
    """

    Attributes
    ----------
    ids: list[str]
    names: list[str]
    """

    ids: list[str] = None
    names: list[str] = None

    def __init__(self_, **kvargs):
        if "ids" in kvargs:
            if type(kvargs["ids"]).__name__ is self_.__annotations__["ids"]:
                self_.ids = kvargs["ids"]
            else:
                self_.ids = kvargs["ids"]
        if "names" in kvargs:
            if type(kvargs["names"]).__name__ is self_.__annotations__["names"]:
                self_.names = kvargs["names"]
            else:
                self_.names = kvargs["names"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class Share:
    """

    Attributes
    ----------
    assigneeId: str
      The userId or groupId based on the type.
    id: str
    resourceId: str
      The ID of the shared resource.
    resourceType: str
      The type of the shared resource.
    spaceId: str
    tenantId: str
    type: str
    createdAt: str
    createdBy: str
      The ID of the user who created the share.
    links: object
    resourceName: str
      The name of the shared resource.
    roles: list[ShareRoleType]
      The roles assigned to the assigneeId.
    updatedAt: str
    updatedBy: str
      The ID of the user who updated the share.
    """

    assigneeId: str = None
    id: str = None
    resourceId: str = None
    resourceType: str = None
    spaceId: str = None
    tenantId: str = None
    type: str = None
    createdAt: str = None
    createdBy: str = None
    links: object = None
    resourceName: str = None
    roles: list[ShareRoleType] = None
    updatedAt: str = None
    updatedBy: str = None

    def __init__(self_, **kvargs):
        if "assigneeId" in kvargs:
            if (
                type(kvargs["assigneeId"]).__name__
                is self_.__annotations__["assigneeId"]
            ):
                self_.assigneeId = kvargs["assigneeId"]
            else:
                self_.assigneeId = kvargs["assigneeId"]
        if "id" in kvargs:
            if type(kvargs["id"]).__name__ is self_.__annotations__["id"]:
                self_.id = kvargs["id"]
            else:
                self_.id = kvargs["id"]
        if "resourceId" in kvargs:
            if (
                type(kvargs["resourceId"]).__name__
                is self_.__annotations__["resourceId"]
            ):
                self_.resourceId = kvargs["resourceId"]
            else:
                self_.resourceId = kvargs["resourceId"]
        if "resourceType" in kvargs:
            if (
                type(kvargs["resourceType"]).__name__
                is self_.__annotations__["resourceType"]
            ):
                self_.resourceType = kvargs["resourceType"]
            else:
                self_.resourceType = kvargs["resourceType"]
        if "spaceId" in kvargs:
            if type(kvargs["spaceId"]).__name__ is self_.__annotations__["spaceId"]:
                self_.spaceId = kvargs["spaceId"]
            else:
                self_.spaceId = kvargs["spaceId"]
        if "tenantId" in kvargs:
            if type(kvargs["tenantId"]).__name__ is self_.__annotations__["tenantId"]:
                self_.tenantId = kvargs["tenantId"]
            else:
                self_.tenantId = kvargs["tenantId"]
        if "type" in kvargs:
            if type(kvargs["type"]).__name__ is self_.__annotations__["type"]:
                self_.type = kvargs["type"]
            else:
                self_.type = kvargs["type"]
        if "createdAt" in kvargs:
            if type(kvargs["createdAt"]).__name__ is self_.__annotations__["createdAt"]:
                self_.createdAt = kvargs["createdAt"]
            else:
                self_.createdAt = kvargs["createdAt"]
        if "createdBy" in kvargs:
            if type(kvargs["createdBy"]).__name__ is self_.__annotations__["createdBy"]:
                self_.createdBy = kvargs["createdBy"]
            else:
                self_.createdBy = kvargs["createdBy"]
        if "links" in kvargs:
            if type(kvargs["links"]).__name__ is self_.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = kvargs["links"]
        if "resourceName" in kvargs:
            if (
                type(kvargs["resourceName"]).__name__
                is self_.__annotations__["resourceName"]
            ):
                self_.resourceName = kvargs["resourceName"]
            else:
                self_.resourceName = kvargs["resourceName"]
        if "roles" in kvargs:
            if type(kvargs["roles"]).__name__ is self_.__annotations__["roles"]:
                self_.roles = kvargs["roles"]
            else:
                self_.roles = [ShareRoleType(**e) for e in kvargs["roles"]]
        if "updatedAt" in kvargs:
            if type(kvargs["updatedAt"]).__name__ is self_.__annotations__["updatedAt"]:
                self_.updatedAt = kvargs["updatedAt"]
            else:
                self_.updatedAt = kvargs["updatedAt"]
        if "updatedBy" in kvargs:
            if type(kvargs["updatedBy"]).__name__ is self_.__annotations__["updatedBy"]:
                self_.updatedBy = kvargs["updatedBy"]
            else:
                self_.updatedBy = kvargs["updatedBy"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class ShareCreate:
    """

    Attributes
    ----------
    assigneeId: str
      The userId or groupId based on the type.
    resourceId: str
      The resource id for the shared item.
    resourceType: str
      The resource type for the shared item.
    roles: list[ShareRoleType]
      The roles assigned to the assigneeId.
    type: str
    """

    assigneeId: str = None
    resourceId: str = None
    resourceType: str = None
    roles: list[ShareRoleType] = None
    type: str = None

    def __init__(self_, **kvargs):
        if "assigneeId" in kvargs:
            if (
                type(kvargs["assigneeId"]).__name__
                is self_.__annotations__["assigneeId"]
            ):
                self_.assigneeId = kvargs["assigneeId"]
            else:
                self_.assigneeId = kvargs["assigneeId"]
        if "resourceId" in kvargs:
            if (
                type(kvargs["resourceId"]).__name__
                is self_.__annotations__["resourceId"]
            ):
                self_.resourceId = kvargs["resourceId"]
            else:
                self_.resourceId = kvargs["resourceId"]
        if "resourceType" in kvargs:
            if (
                type(kvargs["resourceType"]).__name__
                is self_.__annotations__["resourceType"]
            ):
                self_.resourceType = kvargs["resourceType"]
            else:
                self_.resourceType = kvargs["resourceType"]
        if "roles" in kvargs:
            if type(kvargs["roles"]).__name__ is self_.__annotations__["roles"]:
                self_.roles = kvargs["roles"]
            else:
                self_.roles = [ShareRoleType(**e) for e in kvargs["roles"]]
        if "type" in kvargs:
            if type(kvargs["type"]).__name__ is self_.__annotations__["type"]:
                self_.type = kvargs["type"]
            else:
                self_.type = kvargs["type"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class SharePatch:
    """

    Attributes
    ----------
    """

    def __init__(self_, **kvargs):
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class ShareRoleType:
    """
    Supported roles by space type:
    - Shared: consumer
    - Managed: consumer, contributor

    Attributes
    ----------
    """

    def __init__(self_, **kvargs):
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class Shares:
    """

    Attributes
    ----------
    data: list[Share]
    links: object
    meta: object
    """

    data: list[Share] = None
    links: object = None
    meta: object = None

    def __init__(self_, **kvargs):
        if "data" in kvargs:
            if type(kvargs["data"]).__name__ is self_.__annotations__["data"]:
                self_.data = kvargs["data"]
            else:
                self_.data = [Share(**e) for e in kvargs["data"]]
        if "links" in kvargs:
            if type(kvargs["links"]).__name__ is self_.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = kvargs["links"]
        if "meta" in kvargs:
            if type(kvargs["meta"]).__name__ is self_.__annotations__["meta"]:
                self_.meta = kvargs["meta"]
            else:
                self_.meta = kvargs["meta"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class SpaceCreate:
    """

    Attributes
    ----------
    name: str
      The name of the space. Personal spaces do not have a name.
    type: str
      The type of space such as shared, managed, and so on.
    description: str
      The description of the space. Personal spaces do not have a description.
    """

    name: str = None
    type: str = None
    description: str = None

    def __init__(self_, **kvargs):
        if "name" in kvargs:
            if type(kvargs["name"]).__name__ is self_.__annotations__["name"]:
                self_.name = kvargs["name"]
            else:
                self_.name = kvargs["name"]
        if "type" in kvargs:
            if type(kvargs["type"]).__name__ is self_.__annotations__["type"]:
                self_.type = kvargs["type"]
            else:
                self_.type = kvargs["type"]
        if "description" in kvargs:
            if (
                type(kvargs["description"]).__name__
                is self_.__annotations__["description"]
            ):
                self_.description = kvargs["description"]
            else:
                self_.description = kvargs["description"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class SpacePatch:
    """

    Attributes
    ----------
    """

    def __init__(self_, **kvargs):
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class SpaceTypes:
    """
    The distinct types of spaces (shared, managed, and so on).

    Attributes
    ----------
    data: list[str]
    """

    data: list[str] = None

    def __init__(self_, **kvargs):
        if "data" in kvargs:
            if type(kvargs["data"]).__name__ is self_.__annotations__["data"]:
                self_.data = kvargs["data"]
            else:
                self_.data = kvargs["data"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class SpaceUpdate:
    """

    Attributes
    ----------
    description: str
      The description of the space. Personal spaces do not have a description.
    name: str
      The name of the space.
    ownerId: str
      The user ID of the space owner.
    """

    description: str = None
    name: str = None
    ownerId: str = None

    def __init__(self_, **kvargs):
        if "description" in kvargs:
            if (
                type(kvargs["description"]).__name__
                is self_.__annotations__["description"]
            ):
                self_.description = kvargs["description"]
            else:
                self_.description = kvargs["description"]
        if "name" in kvargs:
            if type(kvargs["name"]).__name__ is self_.__annotations__["name"]:
                self_.name = kvargs["name"]
            else:
                self_.name = kvargs["name"]
        if "ownerId" in kvargs:
            if type(kvargs["ownerId"]).__name__ is self_.__annotations__["ownerId"]:
                self_.ownerId = kvargs["ownerId"]
            else:
                self_.ownerId = kvargs["ownerId"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class SpacesClass:
    """

    Attributes
    ----------
    data: list[Space]
    links: object
    meta: object
    """

    data: list[Space] = None
    links: object = None
    meta: object = None

    def __init__(self_, **kvargs):
        if "data" in kvargs:
            if type(kvargs["data"]).__name__ is self_.__annotations__["data"]:
                self_.data = kvargs["data"]
            else:
                self_.data = [Space(**e) for e in kvargs["data"]]
        if "links" in kvargs:
            if type(kvargs["links"]).__name__ is self_.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = kvargs["links"]
        if "meta" in kvargs:
            if type(kvargs["meta"]).__name__ is self_.__annotations__["meta"]:
                self_.meta = kvargs["meta"]
            else:
                self_.meta = kvargs["meta"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


class Spaces:
    def __init__(self, config: Config) -> None:
        self.config = config
        self.auth = Auth(config)

    def get_spaces(
        self,
        type: str = None,
        action: str = None,
        sort: str = None,
        name: str = None,
        ownerId: str = None,
        limit: int = 10,
        next: str = None,
        prev: str = None,
        max_items: int = 10,
    ) -> ListableResource[Space]:
        """
        Retrieves spaces that the current user has access to and match the query.


        type: str
          Type(s) of space to filter. For example, "?type=managed,shared".

        action: str
          Action on space. For example, "?action=publish".

        sort: str
          Field to sort by. Prefix with +/- to indicate asc/desc. For example, "?sort=+name" to sort ascending on Name. Supported fields are "type", "name" and "createdAt".

        name: str
          Space name to search and filter for. Case-insensitive open search with wildcards both as prefix and suffix. For example, "?name=fin" will get "finance", "Final" and "Griffin".

        ownerId: str
          Space ownerId to filter by. For example, "?ownerId=123".

        limit: int
          Maximum number of spaces to return.

        next: str
          The next page cursor. Next links make use of this.

        prev: str
          The previous page cursor. Previous links make use of this.

        Parameters
        ----------
        type: str = None
        action: str = None
        sort: str = None
        name: str = None
        ownerId: str = None
        limit: int = 10
        next: str = None
        prev: str = None
        """
        query_params = {}
        if type is not None:
            query_params["type"] = type
        if action is not None:
            query_params["action"] = action
        if sort is not None:
            query_params["sort"] = sort
        if name is not None:
            query_params["name"] = name
        if ownerId is not None:
            query_params["ownerId"] = ownerId
        if limit is not None:
            query_params["limit"] = limit
        if next is not None:
            query_params["next"] = next
        if prev is not None:
            query_params["prev"] = prev

        response = self.auth.rest(
            path="/spaces",
            method="GET",
            params=query_params,
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=Space,
            auth=self.auth,
            path="/spaces",
            max_items=max_items,
            query_params=query_params,
        )

    def create(self, data: SpaceCreate) -> Space:
        """
        Creates a space.


        Parameters
        ----------
        data: SpaceCreate
        """

        try:
            data = asdict(data)
        except:
            data = data

        response = self.auth.rest(
            path="/spaces",
            method="POST",
            params={},
            data=data,
        )
        obj = Space(**response.json())
        obj.auth = self.auth
        return obj

    def create_filters(
        self, data: FilterSpaces, max_items: int = 10
    ) -> ListableResource[Space]:
        """
        Experimental
        Retrieves spaces that the current user has access to with provided space IDs or names.


        Parameters
        ----------
        data: FilterSpaces
        """
        warnings.warn("create_filters is experimental", UserWarning, stacklevel=2)

        try:
            data = asdict(data)
        except:
            data = data

        response = self.auth.rest(
            path="/spaces/filter",
            method="POST",
            params={},
            data=data,
        )
        return ListableResource(
            response=response.json(),
            cls=Space,
            auth=self.auth,
            path="/spaces/filter",
            max_items=max_items,
            query_params={},
        )

    def get_types(self, max_items: int = 10) -> ListableResource[str]:
        """
        Gets a list of distinct space types.


        Parameters
        ----------
        """

        response = self.auth.rest(
            path="/spaces/types",
            method="GET",
            params={},
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=None,
            auth=self.auth,
            path="/spaces/types",
            max_items=max_items,
            query_params={},
        )

    def get(self, spaceId: str) -> Space:
        """
        Retrieves a single space by ID.


        spaceId: str
          The ID of the space to retrieve.

        Parameters
        ----------
        spaceId: str
        """

        response = self.auth.rest(
            path="/spaces/{spaceId}".replace("{spaceId}", spaceId),
            method="GET",
            params={},
            data=None,
        )
        obj = Space(**response.json())
        obj.auth = self.auth
        return obj

    def delete_assignment(self, spaceId: str, assignmentId: str) -> None:
        """
        Deletes an assignment.


        spaceId: str
          The ID of the space of the assignment.

        assignmentId: str
          The ID of the assignment to delete.

        Parameters
        ----------
        spaceId: str
        assignmentId: str
        """

        self.auth.rest(
            path="/spaces/{spaceId}/assignments/{assignmentId}".replace(
                "{spaceId}", spaceId
            ).replace("{assignmentId}", assignmentId),
            method="DELETE",
            params={},
            data=None,
        )

    def get_assignment(self, spaceId: str, assignmentId: str) -> Assignment:
        """
        Retrieves a single assignment by ID.


        spaceId: str
          The ID of the space of the assignment.

        assignmentId: str
          The ID of the assignment to retrieve.

        Parameters
        ----------
        spaceId: str
        assignmentId: str
        """

        response = self.auth.rest(
            path="/spaces/{spaceId}/assignments/{assignmentId}".replace(
                "{spaceId}", spaceId
            ).replace("{assignmentId}", assignmentId),
            method="GET",
            params={},
            data=None,
        )
        obj = Assignment(**response.json())
        obj.auth = self.auth
        return obj

    def set_assignment(
        self, spaceId: str, assignmentId: str, data: AssignmentUpdate
    ) -> Assignment:
        """
        Experimental
        Updates a single assignment by ID. The complete list of roles must be provided.


        spaceId: str
          The ID of the space of the assignment.

        assignmentId: str
          The ID of the assignment to update.

        Parameters
        ----------
        spaceId: str
        assignmentId: str
        data: AssignmentUpdate
        """
        warnings.warn("set_assignment is experimental", UserWarning, stacklevel=2)

        try:
            data = asdict(data)
        except:
            data = data

        response = self.auth.rest(
            path="/spaces/{spaceId}/assignments/{assignmentId}".replace(
                "{spaceId}", spaceId
            ).replace("{assignmentId}", assignmentId),
            method="PUT",
            params={},
            data=data,
        )
        obj = Assignment(**response.json())
        obj.auth = self.auth
        return obj

    def delete_share(self, spaceId: str, shareId: str) -> None:
        """
        Experimental
        Deletes a Share.


        spaceId: str
          The ID of the space to which the share belongs.

        shareId: str
          The ID of the share to delete.

        Parameters
        ----------
        spaceId: str
        shareId: str
        """
        warnings.warn("delete_share is experimental", UserWarning, stacklevel=2)

        self.auth.rest(
            path="/spaces/{spaceId}/shares/{shareId}".replace(
                "{spaceId}", spaceId
            ).replace("{shareId}", shareId),
            method="DELETE",
            params={},
            data=None,
        )

    def get_share(self, spaceId: str, shareId: str) -> Share:
        """
        Experimental
        Retrieves a single share by ID.


        spaceId: str
          The ID of the space to which the share belongs.

        shareId: str
          The ID of the share to retrieve.

        Parameters
        ----------
        spaceId: str
        shareId: str
        """
        warnings.warn("get_share is experimental", UserWarning, stacklevel=2)

        response = self.auth.rest(
            path="/spaces/{spaceId}/shares/{shareId}".replace(
                "{spaceId}", spaceId
            ).replace("{shareId}", shareId),
            method="GET",
            params={},
            data=None,
        )
        obj = Share(**response.json())
        obj.auth = self.auth
        return obj

    def patch_share(self, spaceId: str, shareId: str, data: SharePatch) -> Share:
        """
        Experimental
        Patches (updates) a share (partially).


        spaceId: str
          The ID of the space to which the share belongs.

        shareId: str
          The ID of the share to update.

        Parameters
        ----------
        spaceId: str
        shareId: str
        data: SharePatch
        """
        warnings.warn("patch_share is experimental", UserWarning, stacklevel=2)

        try:
            data = asdict(data)
        except:
            data = data

        response = self.auth.rest(
            path="/spaces/{spaceId}/shares/{shareId}".replace(
                "{spaceId}", spaceId
            ).replace("{shareId}", shareId),
            method="PATCH",
            params={},
            data=data,
        )
        obj = Share(**response.json())
        obj.auth = self.auth
        return obj



# This is spectacularly generated code by spectacular v0.0.0 based on
# Data Files API 1.30.0

from __future__ import annotations

import io
import warnings
from dataclasses import dataclass

from ..auth import Auth, Config


@dataclass
class DataFileUploadResponse:
    """

    Attributes
    ----------
    createdDate: str
      The date that the uploaded file was created.
    id: str
      The ID for the uploaded file.
    name: str
      The name of the uploaded file.
    ownerId: str
      The 'owner' of a file is the user who last uploaded the file's content.
    size: int
      The size of the uploaded file, in bytes.
    appId: str
      If this file is bound to the lifecycle of a specific app, this is the ID of this app.
    modifiedDate: str
      The date that the updated file was last modified.
    spaceId: str
      If the file was uploaded to a team space, this is the ID of that space.
    """

    createdDate: str = None
    id: str = None
    name: str = None
    ownerId: str = None
    size: int = None
    appId: str = None
    modifiedDate: str = None
    spaceId: str = None

    def __init__(self_, **kvargs):
        if "createdDate" in kvargs:
            if (
                type(kvargs["createdDate"]).__name__
                is self_.__annotations__["createdDate"]
            ):
                self_.createdDate = kvargs["createdDate"]
            else:
                self_.createdDate = kvargs["createdDate"]
        if "id" in kvargs:
            if type(kvargs["id"]).__name__ is self_.__annotations__["id"]:
                self_.id = kvargs["id"]
            else:
                self_.id = kvargs["id"]
        if "name" in kvargs:
            if type(kvargs["name"]).__name__ is self_.__annotations__["name"]:
                self_.name = kvargs["name"]
            else:
                self_.name = kvargs["name"]
        if "ownerId" in kvargs:
            if type(kvargs["ownerId"]).__name__ is self_.__annotations__["ownerId"]:
                self_.ownerId = kvargs["ownerId"]
            else:
                self_.ownerId = kvargs["ownerId"]
        if "size" in kvargs:
            if type(kvargs["size"]).__name__ is self_.__annotations__["size"]:
                self_.size = kvargs["size"]
            else:
                self_.size = kvargs["size"]
        if "appId" in kvargs:
            if type(kvargs["appId"]).__name__ is self_.__annotations__["appId"]:
                self_.appId = kvargs["appId"]
            else:
                self_.appId = kvargs["appId"]
        if "modifiedDate" in kvargs:
            if (
                type(kvargs["modifiedDate"]).__name__
                is self_.__annotations__["modifiedDate"]
            ):
                self_.modifiedDate = kvargs["modifiedDate"]
            else:
                self_.modifiedDate = kvargs["modifiedDate"]
        if "spaceId" in kvargs:
            if type(kvargs["spaceId"]).__name__ is self_.__annotations__["spaceId"]:
                self_.spaceId = kvargs["spaceId"]
            else:
                self_.spaceId = kvargs["spaceId"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)

    def get_connection(self) -> ConnectionsResponse:
        """
        Experimental
        Get the built-in connections used by the engine to load/write data files given a connection ID.

        Parameters
        ----------
        """
        warnings.warn("get_connection is experimental", UserWarning, stacklevel=2)

        response = self.auth.rest(
            path="/qix-datafiles/connections/{id}".replace("{id}", self.id),
            method="GET",
            params={},
            data=None,
        )
        obj = ConnectionsResponse(**response.json())
        obj.auth = self.auth
        return obj

    def change_owner(self, ownerId: str = None) -> None:
        """
        Experimental
        Change the owner of an existing data file.  This is primarily and admin-type of operation.  In general, the
        owner of a data file is implicitly set as part of a data file upload.  For data files which reside in a
        personal space, changing the owner has the effect of moving the data file to te new owner's personal space.

        Parameters
        ----------
        ownerId: str = None
        """
        warnings.warn("change-owner is experimental", UserWarning, stacklevel=2)
        query_params = {}
        if ownerId is not None:
            query_params["ownerId"] = ownerId

        self.auth.rest(
            path="/qix-datafiles/{id}/actions/change-owner".replace("{id}", self.id),
            method="POST",
            params=query_params,
            data=None,
        )

    def change_space(self, spaceId: str = None) -> None:
        """
        Experimental
        Change the space that an existing data file resides in.  This is to allow for a separate admin-type of
        operation that is more global in terms of access in cases where admin users may not explicitly have been
        granted full access to a given space within the declared space-level permissions.  If the space ID is set
        to null, then the datafile will end up residing in the personal space of the user who is the owner of the
        file.

        Parameters
        ----------
        spaceId: str = None
        """
        warnings.warn("change-space is experimental", UserWarning, stacklevel=2)
        query_params = {}
        if spaceId is not None:
            query_params["spaceId"] = spaceId

        self.auth.rest(
            path="/qix-datafiles/{id}/actions/change-space".replace("{id}", self.id),
            method="POST",
            params=query_params,
            data=None,
        )

    def delete(self) -> None:
        """
        Experimental
        Delete the specified data file.

        Parameters
        ----------
        """
        warnings.warn("delete is experimental", UserWarning, stacklevel=2)

        self.auth.rest(
            path="/qix-datafiles/{id}".replace("{id}", self.id),
            method="DELETE",
            params={},
            data=None,
        )

    def set(
        self,
        Data: io.BufferedReader = None,
        Name: str = None,
        AppId: str = None,
        ConnectionId: str = None,
        SourceId: str = None,
        TempContentFileId: str = None,
    ) -> DataFileUploadResponse:
        """
        Experimental
        Re-upload an existing data file.

        Parameters
        ----------
        Name: str = None
        AppId: str = None
        ConnectionId: str = None
        SourceId: str = None
        TempContentFileId: str = None
        """
        warnings.warn("set is experimental", UserWarning, stacklevel=2)
        query_params = {}
        if Name is not None:
            query_params["Name"] = Name
        if AppId is not None:
            query_params["AppId"] = AppId
        if ConnectionId is not None:
            query_params["ConnectionId"] = ConnectionId
        if SourceId is not None:
            query_params["SourceId"] = SourceId
        if TempContentFileId is not None:
            query_params["TempContentFileId"] = TempContentFileId

        files_dict = {}
        files_dict["Data"] = Data

        response = self.auth.rest(
            path="/qix-datafiles/{id}".replace("{id}", self.id),
            method="PUT",
            params=query_params,
            data=None,
            files=files_dict,
        )
        self.__init__(**response.json())
        return self


@dataclass
class ConnectionsResponse:
    """

    Attributes
    ----------
    connectStatement: str
      The connect statement that will be passed to the connector when invoked.
    id: str
    name: str
      The name of the connection.
    type: str
      The type of the connection.
    spaceId: str
      The team space that the given connection is associated with. If null, the connection is not associated
      with any specific team space.
    """

    connectStatement: str = None
    id: str = None
    name: str = None
    type: str = None
    spaceId: str = None

    def __init__(self_, **kvargs):
        if "connectStatement" in kvargs:
            if (
                type(kvargs["connectStatement"]).__name__
                is self_.__annotations__["connectStatement"]
            ):
                self_.connectStatement = kvargs["connectStatement"]
            else:
                self_.connectStatement = kvargs["connectStatement"]
        if "id" in kvargs:
            if type(kvargs["id"]).__name__ is self_.__annotations__["id"]:
                self_.id = kvargs["id"]
            else:
                self_.id = kvargs["id"]
        if "name" in kvargs:
            if type(kvargs["name"]).__name__ is self_.__annotations__["name"]:
                self_.name = kvargs["name"]
            else:
                self_.name = kvargs["name"]
        if "type" in kvargs:
            if type(kvargs["type"]).__name__ is self_.__annotations__["type"]:
                self_.type = kvargs["type"]
            else:
                self_.type = kvargs["type"]
        if "spaceId" in kvargs:
            if type(kvargs["spaceId"]).__name__ is self_.__annotations__["spaceId"]:
                self_.spaceId = kvargs["spaceId"]
            else:
                self_.spaceId = kvargs["spaceId"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class DataFileDeleteRecordResponse:
    """

    Attributes
    ----------
    createdByUser: str
      The user that deleted this datafile.
    deletedDate: str
      The date that the file was deleted.
    fileMetadataId: str
      The unique ID of the file that was deleted.
    name: str
      File Name
    tenantId: str
      The tenant that this file is scoped to.
    spaceId: str
      If not null, the ID of the space that the uploaded file will reside in. If null, this implies that the
      file is stored in the user-private area (DataFiles)
    userId: str
      If the file resided in a user's personal space, the user that this file is scoped to.
    """

    createdByUser: str = None
    deletedDate: str = None
    fileMetadataId: str = None
    name: str = None
    tenantId: str = None
    spaceId: str = None
    userId: str = None

    def __init__(self_, **kvargs):
        if "createdByUser" in kvargs:
            if (
                type(kvargs["createdByUser"]).__name__
                is self_.__annotations__["createdByUser"]
            ):
                self_.createdByUser = kvargs["createdByUser"]
            else:
                self_.createdByUser = kvargs["createdByUser"]
        if "deletedDate" in kvargs:
            if (
                type(kvargs["deletedDate"]).__name__
                is self_.__annotations__["deletedDate"]
            ):
                self_.deletedDate = kvargs["deletedDate"]
            else:
                self_.deletedDate = kvargs["deletedDate"]
        if "fileMetadataId" in kvargs:
            if (
                type(kvargs["fileMetadataId"]).__name__
                is self_.__annotations__["fileMetadataId"]
            ):
                self_.fileMetadataId = kvargs["fileMetadataId"]
            else:
                self_.fileMetadataId = kvargs["fileMetadataId"]
        if "name" in kvargs:
            if type(kvargs["name"]).__name__ is self_.__annotations__["name"]:
                self_.name = kvargs["name"]
            else:
                self_.name = kvargs["name"]
        if "tenantId" in kvargs:
            if type(kvargs["tenantId"]).__name__ is self_.__annotations__["tenantId"]:
                self_.tenantId = kvargs["tenantId"]
            else:
                self_.tenantId = kvargs["tenantId"]
        if "spaceId" in kvargs:
            if type(kvargs["spaceId"]).__name__ is self_.__annotations__["spaceId"]:
                self_.spaceId = kvargs["spaceId"]
            else:
                self_.spaceId = kvargs["spaceId"]
        if "userId" in kvargs:
            if type(kvargs["userId"]).__name__ is self_.__annotations__["userId"]:
                self_.userId = kvargs["userId"]
            else:
                self_.userId = kvargs["userId"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class DataFileModificationResponse:
    """

    Attributes
    ----------
    createdByUser: str
      The ID of the user that created the file.
    createdDate: str
      The date that the uploaded file was created.
    id: str
      The ID for the uploaded file.
    name: str
      The name of the uploaded file.
    ownerId: str
      The 'owner' of a file is the user who last uploaded the file's content.
    size: int
      The size of the uploaded file, in bytes.
    tenantId: str
      The ID of the tenant that this file is scoped to.
    appId: str
      If this file is bound to the lifecycle of a specific app, this is the ID of this app.
    modifiedByUser: str
      The ID of the user that last modified the file.
    modifiedDate: str
      The date that the updated file was last modified.
    spaceId: str
      If the file was uploaded to a team space, this is the ID of that space.
    userId: str
      If the file resides in that user's personal space, the ID of the user that this file is scoped to.
    """

    createdByUser: str = None
    createdDate: str = None
    id: str = None
    name: str = None
    ownerId: str = None
    size: int = None
    tenantId: str = None
    appId: str = None
    modifiedByUser: str = None
    modifiedDate: str = None
    spaceId: str = None
    userId: str = None

    def __init__(self_, **kvargs):
        if "createdByUser" in kvargs:
            if (
                type(kvargs["createdByUser"]).__name__
                is self_.__annotations__["createdByUser"]
            ):
                self_.createdByUser = kvargs["createdByUser"]
            else:
                self_.createdByUser = kvargs["createdByUser"]
        if "createdDate" in kvargs:
            if (
                type(kvargs["createdDate"]).__name__
                is self_.__annotations__["createdDate"]
            ):
                self_.createdDate = kvargs["createdDate"]
            else:
                self_.createdDate = kvargs["createdDate"]
        if "id" in kvargs:
            if type(kvargs["id"]).__name__ is self_.__annotations__["id"]:
                self_.id = kvargs["id"]
            else:
                self_.id = kvargs["id"]
        if "name" in kvargs:
            if type(kvargs["name"]).__name__ is self_.__annotations__["name"]:
                self_.name = kvargs["name"]
            else:
                self_.name = kvargs["name"]
        if "ownerId" in kvargs:
            if type(kvargs["ownerId"]).__name__ is self_.__annotations__["ownerId"]:
                self_.ownerId = kvargs["ownerId"]
            else:
                self_.ownerId = kvargs["ownerId"]
        if "size" in kvargs:
            if type(kvargs["size"]).__name__ is self_.__annotations__["size"]:
                self_.size = kvargs["size"]
            else:
                self_.size = kvargs["size"]
        if "tenantId" in kvargs:
            if type(kvargs["tenantId"]).__name__ is self_.__annotations__["tenantId"]:
                self_.tenantId = kvargs["tenantId"]
            else:
                self_.tenantId = kvargs["tenantId"]
        if "appId" in kvargs:
            if type(kvargs["appId"]).__name__ is self_.__annotations__["appId"]:
                self_.appId = kvargs["appId"]
            else:
                self_.appId = kvargs["appId"]
        if "modifiedByUser" in kvargs:
            if (
                type(kvargs["modifiedByUser"]).__name__
                is self_.__annotations__["modifiedByUser"]
            ):
                self_.modifiedByUser = kvargs["modifiedByUser"]
            else:
                self_.modifiedByUser = kvargs["modifiedByUser"]
        if "modifiedDate" in kvargs:
            if (
                type(kvargs["modifiedDate"]).__name__
                is self_.__annotations__["modifiedDate"]
            ):
                self_.modifiedDate = kvargs["modifiedDate"]
            else:
                self_.modifiedDate = kvargs["modifiedDate"]
        if "spaceId" in kvargs:
            if type(kvargs["spaceId"]).__name__ is self_.__annotations__["spaceId"]:
                self_.spaceId = kvargs["spaceId"]
            else:
                self_.spaceId = kvargs["spaceId"]
        if "userId" in kvargs:
            if type(kvargs["userId"]).__name__ is self_.__annotations__["userId"]:
                self_.userId = kvargs["userId"]
            else:
                self_.userId = kvargs["userId"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class QuotaResponse:
    """

    Attributes
    ----------
    allowedExtensions: list[str]
      The allowed file extensions on files that are uploaded.
    allowedInternalExtensions: list[str]
      The allowed file extensions for files that are only used internally by the system (and thus not typically
      shown to end users).
    maxFileSize: int
      Maximum allowable size of an uploaded file.
    maxLargeFileSize: int
      Maximum allowable size for a single uploaded large data file (in bytes). This is a file that was indirectly
      uploaded using the temp content service chunked upload capability.
    maxSize: int
      The maximum aggregate size of all files uploaded by a given user.
    size: int
      The current aggregate size of all files uploaded by a given user. If the current aggregate size is greater
      than the maximum aggregate size, this is a quota violation.
    """

    allowedExtensions: list[str] = None
    allowedInternalExtensions: list[str] = None
    maxFileSize: int = None
    maxLargeFileSize: int = None
    maxSize: int = None
    size: int = None

    def __init__(self_, **kvargs):
        if "allowedExtensions" in kvargs:
            if (
                type(kvargs["allowedExtensions"]).__name__
                is self_.__annotations__["allowedExtensions"]
            ):
                self_.allowedExtensions = kvargs["allowedExtensions"]
            else:
                self_.allowedExtensions = kvargs["allowedExtensions"]
        if "allowedInternalExtensions" in kvargs:
            if (
                type(kvargs["allowedInternalExtensions"]).__name__
                is self_.__annotations__["allowedInternalExtensions"]
            ):
                self_.allowedInternalExtensions = kvargs["allowedInternalExtensions"]
            else:
                self_.allowedInternalExtensions = kvargs["allowedInternalExtensions"]
        if "maxFileSize" in kvargs:
            if (
                type(kvargs["maxFileSize"]).__name__
                is self_.__annotations__["maxFileSize"]
            ):
                self_.maxFileSize = kvargs["maxFileSize"]
            else:
                self_.maxFileSize = kvargs["maxFileSize"]
        if "maxLargeFileSize" in kvargs:
            if (
                type(kvargs["maxLargeFileSize"]).__name__
                is self_.__annotations__["maxLargeFileSize"]
            ):
                self_.maxLargeFileSize = kvargs["maxLargeFileSize"]
            else:
                self_.maxLargeFileSize = kvargs["maxLargeFileSize"]
        if "maxSize" in kvargs:
            if type(kvargs["maxSize"]).__name__ is self_.__annotations__["maxSize"]:
                self_.maxSize = kvargs["maxSize"]
            else:
                self_.maxSize = kvargs["maxSize"]
        if "size" in kvargs:
            if type(kvargs["size"]).__name__ is self_.__annotations__["size"]:
                self_.size = kvargs["size"]
            else:
                self_.size = kvargs["size"]
        for k, v in kvargs.items():
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


class QixDatafiles:
    def __init__(self, config: Config) -> None:
        self.config = config
        self.auth = Auth(config)

    def get_qix_datafiles(
        self,
        connectionId: str = None,
        appId: str = None,
        name: str = None,
        top: int = None,
        skip: int = None,
        ownerId: str = None,
        allowInternalFiles: bool = None,
    ) -> list[DataFileUploadResponse]:
        """
        Experimental
        Get descriptive info for the specified data files.


        connectionId: str
          Return files that reside in the space referenced by the specified DataFiles connection. If this parameter is not specified, the user's private space is implied.

        appId: str
          Only return files scoped to the specified app. If this parameter is not specified, only files that are not scoped to any app are returned. "*" implies all app-scoped files are returned.

        name: str
          Filter the list of files returned to the given file name.

        top: int
          If present, restrict the number of returned items to this value.

        skip: int
          If present, skip this number of the returned values in the result set (facilitates paging).

        ownerId: str
          If present, fetch the datafiles for the specified owner. If a connectionId is specified in this case, the returned list is constrained to the specified space. If connectionId is not specified, then all files owned by the specified user are returned regardless of the personal space that a given file resides in.

        allowInternalFiles: bool
          If set to false, do not return datafiles with internal extension else return all the datafiles.

        Parameters
        ----------
        connectionId: str = None
        appId: str = None
        name: str = None
        top: int = None
        skip: int = None
        ownerId: str = None
        allowInternalFiles: bool = None
        """
        warnings.warn("get_qix-datafiles is experimental", UserWarning, stacklevel=2)
        query_params = {}
        if connectionId is not None:
            query_params["connectionId"] = connectionId
        if appId is not None:
            query_params["appId"] = appId
        if name is not None:
            query_params["name"] = name
        if top is not None:
            query_params["top"] = top
        if skip is not None:
            query_params["skip"] = skip
        if ownerId is not None:
            query_params["ownerId"] = ownerId
        if allowInternalFiles is not None:
            query_params["allowInternalFiles"] = allowInternalFiles

        response = self.auth.rest(
            path="/qix-datafiles",
            method="GET",
            params=query_params,
            data=None,
        )
        return [DataFileUploadResponse(**e) for e in response.json()]

    def create(
        self,
        Data: io.BufferedReader = None,
        Name: str = None,
        AppId: str = None,
        ConnectionId: str = None,
        SourceId: str = None,
        TempContentFileId: str = None,
    ) -> DataFileUploadResponse:
        """
        Experimental
        Upload a new data file.  For the upload of a new file, the 'name' parameter is required.


        Name: str
          Name that will be given to the uploaded file. If this name is different than the name used when the file
          was last POSTed or PUT, this will result in a rename of the file. It should be noted that the '/' character
          in a datafile name indicates a 'path' separator in a logical folder hierarchy for the name. Names which
          contain '/'s should be used with the assumption that a logical 'folder hierarchy' is being defined for the
          full pathname of that file. IE, '/' is a significant character in the datafile name, and may impact the
          behavior of future APIs which take this folder hierarchy into account.

        AppId: str
          If this file should be bound to the lifecycle of a specific app, this is the ID of this app.

        ConnectionId: str
          If present, this is the DataFiles connection that the upload should occur in the context of. If absent,
          the default is that the upload will occur in the context of the MyDataFiles connection. If the DataFiles
          connection is different from the one specified when the file was last POSTed or PUT, this will result in
          a logical move of this file into the new space.

        SourceId: str
          If a SourceId is specified, this is the ID of the existing datafile whose content should be copied into
          the specified datafile. That is, instead of the file content being specified in the Data element,
          it is effectively copied from an existing. previously uploaded file.

        TempContentFileId: str
          If a TempContentFileId is specified, this is the ID of a previously uploaded temporary content file whose
          content should be copied into the specified datafile. That is, instead of the file content being specified
          in the Data element, it is effectively copied from an existing. previously uploaded file. The expectation
          is that this file was previously uploaded to the temporoary content service, and the ID specified here is
          the one returned from the temp content upload request.

        Parameters
        ----------
        Name: str = None
        AppId: str = None
        ConnectionId: str = None
        SourceId: str = None
        TempContentFileId: str = None
        """
        warnings.warn("create is experimental", UserWarning, stacklevel=2)
        query_params = {}
        if Name is not None:
            query_params["Name"] = Name
        if AppId is not None:
            query_params["AppId"] = AppId
        if ConnectionId is not None:
            query_params["ConnectionId"] = ConnectionId
        if SourceId is not None:
            query_params["SourceId"] = SourceId
        if TempContentFileId is not None:
            query_params["TempContentFileId"] = TempContentFileId

        files_dict = {}
        files_dict["Data"] = Data

        response = self.auth.rest(
            path="/qix-datafiles",
            method="POST",
            params=query_params,
            data=None,
            files=files_dict,
        )
        obj = DataFileUploadResponse(**response.json())
        obj.auth = self.auth
        return obj

    def get_connections(
        self,
        appId: str = None,
        name: str = None,
        spaceId: str = None,
        personal: bool = None,
    ) -> list[ConnectionsResponse]:
        """
        Experimental
        Get the list of built-in connections used by the engine to load/write data files.  The non-filtered list
        contains a set of hardcoded connections, along with 1 connection per team space that the given user has
        access to.


        appId: str
          Optional. If present, get connections with connection strings which are scoped to the given app ID.

        name: str
          Optional. If present, only return connections with the given name.

        spaceId: str
          Optional. If present, only return the connection which accesses data files in the specified space.

        personal: bool
          Optional. If true, only return the connections which access data in a personal space. Default is false.

        Parameters
        ----------
        appId: str = None
        name: str = None
        spaceId: str = None
        personal: bool = None
        """
        warnings.warn("get_connections is experimental", UserWarning, stacklevel=2)
        query_params = {}
        if appId is not None:
            query_params["appId"] = appId
        if name is not None:
            query_params["name"] = name
        if spaceId is not None:
            query_params["spaceId"] = spaceId
        if personal is not None:
            query_params["personal"] = personal

        response = self.auth.rest(
            path="/qix-datafiles/connections",
            method="GET",
            params=query_params,
            data=None,
        )
        return [ConnectionsResponse(**e) for e in response.json()]

    def get_deletes(
        self,
        deleteStartDate: str = None,
        deleteEndDate: str = None,
        top: int = None,
        skip: int = None,
        allowInternalFiles: bool = None,
    ) -> list[DataFileDeleteRecordResponse]:
        """
        Experimental
        Get descriptive info for data files which have been deleted between the specified start and end dates.  This
        endpoint will enumerate all deleted data files across a given tenant regardless of which (personal or shared) space
        the files reside in.  Note that elevated (service-to-service) priviledges are required to invoke this
        endpoint.


        deleteStartDate: str
          If specified, the returned list will only include datafiles which have been deleted since the
          specified date (inclusive). The date should be specified in Zulu time format (for example: 2020-07-07T20:52:40.8534780Z).

        deleteEndDate: str
          If specified, the returned list will only include datafiles which have been deleted prior to the
          specified data (inclusive). The date should be specified in Zulu time format (for example: 2020-07-07T20:52:40.8534780Z).

        top: int
          If present, restrict the number of returned items to this value.

        skip: int
          If present, skip this number of the returned values in the result set (facilitates paging).

        allowInternalFiles: bool
          If set to false, do not return datafiles with internal extension else return all the datafiles.

        Parameters
        ----------
        deleteStartDate: str = None
        deleteEndDate: str = None
        top: int = None
        skip: int = None
        allowInternalFiles: bool = None
        """
        warnings.warn("get_deletes is experimental", UserWarning, stacklevel=2)
        query_params = {}
        if deleteStartDate is not None:
            query_params["deleteStartDate"] = deleteStartDate
        if deleteEndDate is not None:
            query_params["deleteEndDate"] = deleteEndDate
        if top is not None:
            query_params["top"] = top
        if skip is not None:
            query_params["skip"] = skip
        if allowInternalFiles is not None:
            query_params["allowInternalFiles"] = allowInternalFiles

        response = self.auth.rest(
            path="/qix-datafiles/deletes",
            method="GET",
            params=query_params,
            data=None,
        )
        return [DataFileDeleteRecordResponse(**e) for e in response.json()]

    def get_modifications(
        self,
        modifiedStartDate: str = None,
        modifiedEndDate: str = None,
        top: int = None,
        skip: int = None,
        allowInternalFiles: bool = None,
    ) -> list[DataFileModificationResponse]:
        """
        Experimental
        Get descriptive info for data files which have been modified between the specified start and end dates.  This
        endpoint will enumerate all data files across a given tenant regardless of which (personal or shared) space
        the files reside in.  Note that elevated (service-to-service) priviledges are required to invoke this
        endpoint.


        modifiedStartDate: str
          If specified, the returned list will only include datafiles which have been created or modified since the
          specified date (inclusive). The date should be specified in Zulu time format (for example: 2020-07-07T20:52:40.8534780Z).

        modifiedEndDate: str
          If specified, the returned list will only include datafiles which have been created or modified prior to the
          specified data (inclusive). The date should be specified in Zulu time format (for example: 2020-07-07T20:52:40.8534780Z).

        top: int
          If present, restrict the number of returned items to this value.

        skip: int
          If present, skip this number of the returned values in the result set (facilitates paging).

        allowInternalFiles: bool
          If set to false, do not return datafiles with internal extension else return all the datafiles.

        Parameters
        ----------
        modifiedStartDate: str = None
        modifiedEndDate: str = None
        top: int = None
        skip: int = None
        allowInternalFiles: bool = None
        """
        warnings.warn("get_modifications is experimental", UserWarning, stacklevel=2)
        query_params = {}
        if modifiedStartDate is not None:
            query_params["modifiedStartDate"] = modifiedStartDate
        if modifiedEndDate is not None:
            query_params["modifiedEndDate"] = modifiedEndDate
        if top is not None:
            query_params["top"] = top
        if skip is not None:
            query_params["skip"] = skip
        if allowInternalFiles is not None:
            query_params["allowInternalFiles"] = allowInternalFiles

        response = self.auth.rest(
            path="/qix-datafiles/modifications",
            method="GET",
            params=query_params,
            data=None,
        )
        return [DataFileModificationResponse(**e) for e in response.json()]

    def get_quota(self) -> QuotaResponse:
        """
        Experimental
        Get quota information for the calling user.


        Parameters
        ----------
        """
        warnings.warn("get_quota is experimental", UserWarning, stacklevel=2)

        response = self.auth.rest(
            path="/qix-datafiles/quota",
            method="GET",
            params={},
            data=None,
        )
        obj = QuotaResponse(**response.json())
        obj.auth = self.auth
        return obj

    def get(self, id: str) -> DataFileUploadResponse:
        """
        Experimental
        Get descriptive info for the specified data file.


        id: str
          The ID of the data file.

        Parameters
        ----------
        id: str
        """
        warnings.warn("get is experimental", UserWarning, stacklevel=2)

        response = self.auth.rest(
            path="/qix-datafiles/{id}".replace("{id}", id),
            method="GET",
            params={},
            data=None,
        )
        obj = DataFileUploadResponse(**response.json())
        obj.auth = self.auth
        return obj
